#########
# Author:        rmp
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-08-29 10:35:11 +0100 (Fri, 29 Aug 2008) $
# Id:            $Id: view.pm 2862 2008-08-29 09:35:11Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-2.0/lib/npg_qc/view.pm $
#
package npg_qc::view;
use strict;
use warnings;
use Template;
use base qw(ClearPress::view);
use npg_qc::util;
use Carp;
use English qw(-no_match_vars);
use npg_common::image::scale;

our $VERSION = do { my ($r) = q$LastChangedRevision: 2862 $ =~ /(\d+)/mx; $r; };

sub new {
  my ($class, @args) = @_;
  my $self           = $class->SUPER::new(@args);
  my $util           = $self->util();
  my $username       = 'public';
  my $requestor      = {username => $username};

  $self->util->requestor($requestor);

  return $self;
}

sub authorised {
  my $self = shift;
  return 1;
}

sub decor {
  my ($self, @args) = @_;
  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms || $aspect eq 'read_heatmap' || $aspect eq 'graph') {
    return 0;
  }

  return $self->SUPER::decor(@args);
}

sub content_type {
  my ($self, @args) = @_;
  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms || $aspect eq 'read_heatmap' || $aspect eq 'graph') {
    return 'image/png';
  }

  return $self->SUPER::content_type(@args);
}

sub render {
  my ($self, @args) = @_;

  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms || $aspect eq 'read_heatmap' || $aspect eq 'graph') {
    return $self->$aspect();
  }

  return $self->SUPER::render(@args);
}

sub list_refactor {
  my ($self) = @_;
  my $cgi   = $self->util->cgi();
  my $model = $self->model();
  $model->{id_run} = $cgi->param('id_run');
  return;
}

sub read_scale_png {
  my ($self, $arg_refs) = @_;

  my $scale_img_obj = npg_common::image::scale->new();

  my $png;

  eval {

    $png = $scale_img_obj->plot_scale($arg_refs);

  } or do {

    croak $EVAL_ERROR;

  };

  return $png;
}

1;

__END__

=head1 NAME

npg_qc::view - npg-qc MVC view superclass, derived from ClearPress::View

=head1 VERSION

$LastChangedRevision: 2862 $

=head1 SYNOPSIS

  my $oView = npg_qc::view::<subclass>->new({'util' => $oUtil});
  $oView->model($oModel);

  print $oView->decor()?
    $sw->header()
    :
    q(Content-type: ).$oView->content_type()."\n\n";

  print $oView->render();

  print $oView->decor()?$sw->footer():q();

=head1 DESCRIPTION

View superclass for the NPG-QC MVC application

=head1 SUBROUTINES/METHODS

=head2 new - constructor

  my $oView = npg::view::<subclass>->new({'util' => $oUtil, ...});

=head2 authorised - authorise everybody

=head2 decor

=head2 content_type

=head2 render

=head2 list_refactor - refactor of common code to put id_run from cgi parameters onto the base model for the view

=head2 read_scale_png - returns a scale image for use with heatmaps

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Template
npg::util
npg::model::user
npg::model::usergroup
ClearPress::view
Carp
English
npg_common::image::scale

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Roger Pettett

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
