#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-09-30
# Last Modified: $Date: 2008-10-10 09:45:46 +0100 (Fri, 10 Oct 2008) $
# Id:            $Id: run_graph.pm 3210 2008-10-10 08:45:46Z gq1 $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-2.0/lib/npg_qc/view/run_graph.pm $
#

package npg_qc::view::run_graph;
use strict;
use warnings;
use base qw(npg_qc::view);
use English qw{-no_match_vars};
use Carp qw(confess cluck carp croak);
use Readonly;
use npg_qc::model::run_graph;
use npg_common::image::graph;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3210 $ =~ /(\d+)/mx; $r; };

Readonly our $WIDTH                => 800;
Readonly our $HEIGHT               => 460;
Readonly our $NUM_RUNS_GRAPH       => 100;
Readonly our $MAX_X_LABEL          => 50;

sub decor {
  my ($self, @args) = @_;
  
  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms || $aspect eq 'create_xml') {
    return 0;
  }

  return $self->SUPER::decor(@args);
}

sub content_type {
  my ($self, @args) = @_;

  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms ) {
    return 'image/png';
  }
  
  if($aspect eq 'create_xml' ) {

    return 'application/xml';
  }
  return $self->SUPER::content_type(@args);
}

sub render {
  my ($self, @args) = @_;

  my $aspect = $self->aspect() || q();

  if($aspect =~ /\Aread_.*?png\z/xms ) {
    return $self->$aspect();
  }
  if($aspect eq 'create_xml' ) {

     return $self->create();
  }
  return $self->SUPER::render(@args);
}

sub create {
  my ($self)  = @_;

  my $util    = $self->util();
  my $cgi     = $util->cgi();

  my $content = $cgi->param('POSTDATA');

  if (!$content) {
    $content = $cgi->param('XForms:Model');
  }
  #warn $content;
  my $parser = $util->parser();

  my $parsed_xml;

  eval {
    $parsed_xml = $parser->parse_string($content);
    1;
  } or do {
    croak 'Not well formed xml';
  };

  my $method = $parsed_xml->getElementsByTagName('method');
  $method = lc $method;

  eval {
    $self->allowed_methods($method);
    1;
  } or do {
    croak $EVAL_ERROR
  };

  my $id_run = $parsed_xml->getElementsByTagName('id_run');
  
  if ($id_run) {
    my $end = $id_run->[0]->getAttribute('end');
    if(!$end){
      $end = 1;
    }
    return $self->only_run($id_run, $end);
  }

  return $self->all_runs();

}

sub allowed_methods {
  my ($self, $method) = @_;

  my $allowed_methods = {
    run_graph => 1,
  };

  if (!$allowed_methods->{$method}) {
    croak "This method ($method) is not allowed";
  }

  return 1;
}

sub response_object {
  my ($self, $response) = @_;

  return qq{<?xml version="1.0" encoding="utf-8"?><response>$response</response>};
}

sub only_run {
  my ($self, $id_run, $end) = @_;

  my $model = $self->model();

  my $response;
  eval{
    my $run_graph_model = $model->calculate_one_run($id_run, $end);
    my $actual_id_run = $run_graph_model->id_run();

    $response = "success: run($actual_id_run $end) done for run_graph";
    1;
  } or do{
    $response = "failed: $EVAL_ERROR"
  };

  return $self->response_object($response);
}

sub all_runs {
  my ($self) = @_;
  my $model = $self->model();
  my $response;
  eval{
    my $num_runs_done = $model->calculate_all();

    $response = "success: $num_runs_done runs done for run_graph";
    1;
  } or do{
    $response = "failed: $EVAL_ERROR"
  };
  return $self->response_object($response);
}


sub read_yield_by_run_png {
  my ($self)    = @_;

  my $model     = $self->model();

  my $cgi       = $self->util->cgi();
  
  my $num_runs  = $cgi->param('size');
  if(!$num_runs){
    $num_runs = $NUM_RUNS_GRAPH ;
  }

  my $data  = $model->get_yield_by_run($num_runs);
  
  my $specific_info = {
    data    => $data,
    legend  => undef,
    y_label => 'PF Yield (GBases)',
    title   => 'PF Yield per Run',
    num_runs => $num_runs,
  };

  return $self->draw_graph_run_id_refactor($specific_info);
}

sub read_error_by_run_png {
  my ($self)    = @_;

  my $model     = $self->model();

  my $cgi       = $self->util->cgi();
  
  my $num_runs  = $cgi->param('size');
  if(!$num_runs){
    $num_runs = $NUM_RUNS_GRAPH ;
  }

  my $data  = $model->get_avg_error_by_run($num_runs);
  
  my $specific_info = {
    data    => $data,
    legend  => undef,
    y_label => 'Error Rate %',
    title   => 'Error Rate by Run',
    num_runs => $num_runs,
  };

  return $self->draw_graph_run_id_refactor($specific_info);
}

sub read_cluster_per_tile_by_run_png {
  my ($self)    = @_;

  my $model     = $self->model();

  my $cgi       = $self->util->cgi();
  
  my $num_runs  = $cgi->param('size');
  if(!$num_runs){
    $num_runs = $NUM_RUNS_GRAPH ;
  }

  my $data  = $model->get_cluster_per_tile_by_run($num_runs);
  
  my $specific_info = {
    data    => $data,
    legend  => ['PF Cluster', 'Raw Cluster'],
    y_label => 'Average Cluster Count per Tile',
    title   => 'Average Cluster Count per Tile by Run (All Lanes)',
    num_runs => $num_runs,
  };

  return $self->draw_graph_run_id_refactor($specific_info);
}

sub read_cluster_per_tile_by_run_control_png {
  my ($self)    = @_;

  my $model     = $self->model();

  my $cgi       = $self->util->cgi();
  
  my $num_runs  = $cgi->param('size');
  if(!$num_runs){
    $num_runs = $NUM_RUNS_GRAPH ;
  }

  my $data  = $model->get_cluster_per_tile_control_by_run($num_runs);
  
  my $specific_info = {
    data    => $data,
    legend  => ['PF Cluster', 'Raw Cluster'],
    y_label => 'Average Cluster Count per Tile',
    title   => 'Average Cluster Count per Tile by Run (Control Lanes)',
    num_runs => $num_runs,
  };

  return $self->draw_graph_run_id_refactor($specific_info);
}

sub draw_graph_run_id_refactor {
  my ($self, $specific_info) = @_;

  my $x_label_skip = int $specific_info->{num_runs}/$MAX_X_LABEL;

  my $graph_obj = npg_common::image::graph->new({util => $self->util});

  my $args = {
    'width'        => $WIDTH,
    'height'       => $HEIGHT,
    'title'        => $specific_info->{title},
    'x_label_skip' => $x_label_skip,
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => $specific_info->{y_label},
    'legend'      => $specific_info->{legend},

  };

  return $graph_obj->plotter($specific_info->{data}, $args, q{lines}, undef);
}

1;
__END__
=head1 NAME

npg_qc::view::run_graph

=head1 VERSION

$LastChangedRevision: 3210 $

=head1 SYNOPSIS

  my $o = npg_qc::view::run_graph->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 decor - sets decor to 0, as only jobis to process and respond with xml

=head2 content_type - sets content_type to 'application/xml'

=head2 render - renders view

=head2 create - overall handler to obtain xml and determine what to do with it

=head2 allowed_methods - returns true if xml requested method is ok

=head2 response_object - returns an xml response object with information in it

=head2 all_runs - handler to run the cache_query stuff for all runs in the database which need them

=head2 only_run - handler to run the cache_query stuff for a single run, regenerating if they already exist

=head2 read_yield_by_run_png - generate yield by run png graph of last 100 runs, the number of runs can be passed in

=head2 read_cluster_per_tile_by_run_control_png - generate average cluster count per tile for both RAW and PF by run png graph of last 100 runs, the number of runs can be passed in, this just for control lanes

=head2 read_cluster_per_tile_by_run_png - generate average cluster count per tile for both RAW and PF by run png graph of last 100 runs, the number of runs can be passed in

=head2 read_error_by_run_png - generate average error by run png graph of last 100 runs, the number of runs can be passed in

=head2 draw_graph_run_id_refactor - refactor of common code used by read_*_png

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::view

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
