#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-08-29 10:35:11 +0100 (Fri, 29 Aug 2008) $
# Id:            $Id: signal_mean.pm 2862 2008-08-29 09:35:11Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-2.0/lib/npg_qc/view/signal_mean.pm $
#

package npg_qc::view::signal_mean;
use strict;
use warnings;
use npg_qc::graph;
use base qw(npg_qc::view);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 2862 $ =~ /(\d+)/mx; $r; };

Readonly our $STD_HEIGHT   => 700;
Readonly our $STD_WIDTH    => 504;
Readonly our $STD_Y        => 100;
Readonly our $X_LABEL_SKIP => 5;
Readonly our $THUMB_HEIGHT => 84;
Readonly our $THUMB_WIDTH  => 126;
Readonly our $ALL_CALL_Y   => 15_000;

sub list {
  my ($self) = @_;
  my $util            = $self->util();
  my $model           = $self->model();
  my $cgi             = $util->cgi();
  $model->{rescale_y} = $cgi->param('rescale_y') || $ALL_CALL_Y;
  return $self->list_refactor();
}

sub read_png {
  my ($self)    = @_;
  my $util      = $self->util();
  my $model     = $self->model();
  my $cgi       = $util->cgi();
  my $plot      = $cgi->param('plot');
  my $id_run    = $cgi->param('id_run');
  my $position  = $cgi->param('position');
  my $thumbnail = $cgi->param('thumb');
  my $rescale_y = $cgi->param('rescale_y') || $ALL_CALL_Y;
  my $graph_obj = npg_qc::graph->new({util => $util});

  $model->{id_run}    = $id_run;
  $model->{rescale_y} = $rescale_y;

  my $data = $model->$plot($position);

  my $args = {
   'x_label' => 'Cycle',
   'y_label' => 'Intensity',
   'title'   => ucfirst$plot . " Signal Mean for Run $id_run, Position $position",
   'legend'  => [qw(a c g t)],
   'height'  => $STD_HEIGHT,
   'width'   => $STD_WIDTH,
   'x_label_skip' => $X_LABEL_SKIP,
   'y_min_value'  => 0,
#   'y_max_value'  => $STD_Y,
  };

  if ($plot =~ /all/xms && $rescale_y ne 'self_scale') {
    $args->{y_max_value} = $rescale_y;
  }

  if ($thumbnail eq 'true') {
    $args->{height}  = $THUMB_HEIGHT;
    $args->{width}   = $THUMB_WIDTH;
    $args->{title}   = undef;
    $args->{x_label} = undef;
    $args->{y_label} = undef;
    $args->{legend}  = undef;
  }

  return $graph_obj->plotter(
    $data,
    $args,
    q{},
    1
  );
}

1;
__END__
=head1 NAME

npg_qc::view::signal_mean

=head1 VERSION

$LastChangedRevision: 2862 $

=head1 SYNOPSIS

  my $oSignalMeanView = npg_qc::view::signal_mean->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 list - handles the viewing of the IVC plot page

=head2 read_png - handles generating the png image base on parameters sent

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::view

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
