#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-12
# Last Modified: $Date: 2008-10-10 09:45:46 +0100 (Fri, 10 Oct 2008) $
# Id:            $Id: util.pm 3210 2008-10-10 08:45:46Z gq1 $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-2.0/t/api/util.pm $
#
package t::api::util;
use strict;
use warnings;
use Class::Std;
use base qw(npg_qc::api::util);
use Carp;
use DateTime;
use CGI;
use English qw(-no_match_vars);
use Test::More;
use HTML::PullParser;
use YAML qw(LoadFile);
use MIME::Parser;
use MIME::Lite;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3210 $ =~ /(\d+)/mx; $r; };

$ENV{HTTP_HOST}     = 'test.npg.com';
$ENV{DOCUMENT_ROOT} = './htdocs';
$ENV{SCRIPT_NAME}   = '/cgi-bin/npg_qc';
$ENV{dev}           = 'test';

{
  my %fixtures_of         :ATTR( init_arg => 'fixtures', :get<fixtures>, :set<fixtures>);
  my %email_annotation_of :ATTR( :get<email_annotation>, :set<email_annotation>);
  my %email_subject_of    :ATTR( :get<email_subject>, :set<email_subject>);
  my %email_to_of         :ATTR( :get<email_to>, :set<email_to>);
  my %email_bcc_of        :ATTR( :get<email_bcc>, :set<email_bcc>);
  my %email_from_of       :ATTR( :get<email_from>, :set<email_from>);
  my %dbh_of              :ATTR( init_arg => 'dbh', :get<dbh>, :set<dbh>);
}

sub BUILD {
  my ($self) = @_;

  if($self->get_fixtures()) {
    $self->load_fixtures();
  }

  return;
}


sub load_fixtures {
  my $self = shift;

#  if($self->dbsection() ne 'test') {
#    croak "dbsection is set to @{[$self->dbsection()]} which is not the same as 'test'. Refusing to go ahead!";
#  }

  #########
  # build table definitions
  #
  if(!-e "data/schema.txt") {
    croak "Could not find data/schema.txt";
  }

  carp 'Loading data/schema.txt';
  my $cmd = sprintf q(cat data/schema.txt | mysql -u%s -P%s -h%s %s),
                    'nqcadmin',
		    3309,
		    'mcs4a',
		    'npgqct';

  carp "Executing: $cmd";
  open my $fh, q(-|), $cmd or croak $ERRNO;
  while(<$fh>) {
    print;
  }
  close $fh or croak $ERRNO;

  #########
  # populate test data
  #
  carp 'Loading fixtures';

  opendir my $dh, q(t/data/fixtures) or croak "Could not open t/data/fixtures";
  my @fixtures = sort grep { /\d+\-[a-z\d_]+\.yml$/mix } readdir $dh;
  closedir $dh;

  my $dbh = $self->get_dbh();
  for my $fx (@fixtures) {
    my $yml     = LoadFile("t/data/fixtures/$fx");
    my ($table) = $fx =~ /\-([a-z\d_]+)/mix;
    carp "+- Loading $fx into $table";
    my $row1    = $yml->[0];
    my @fields  = keys %{$row1};
    my $query   = qq(INSERT INTO $table (@{[join q(, ), @fields]}) VALUES (@{[join q(,), map { q(?) } @fields]}));

    for my $row (@{$yml}) {
      $dbh->do($query, {}, map { $row->{$_} } @fields);
    }
    $dbh->commit();
  }

  return;
  

}

sub date_today {
  my ($self,$database) = @_;
  my $dt               = DateTime->now();
  if ($database && $database eq 'mysql') {
    $dt =~ s/T/ /gxms;
    return $dt;
  }
  return $self->_dt2ora($dt);
}

sub _dt2ora {
  my ($self, $dt) = @_;
  my $monthname = [qw(JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC)]->[$dt->month-1];
  return sprintf q(%02d-%3s-%02d), $dt->day, $monthname, $dt->year-2000;
}

sub prof {
  my $self = shift;
  $self->{'prof'} ||= Website::Utilities::Profiler->new();
  return $self->{'prof'};
}

sub profiler {
  my $self = shift;
  $self->{'prof'} ||= Website::Utilities::Profiler->new();
  return $self->{'prof'};
}

sub data_root {
  return './data';
}

sub cgi {
  my $self = shift;
  $self->{'cgi'} ||= CGI->new();
  return $self->{'cgi'};
}

sub ora_dbname {
  return 'mock';
}

sub rendered {
  my ($self, $tt_name) = @_;
  local $RS = undef;
  open my $fh, q(<), $tt_name or croak "Error opening $tt_name: $ERRNO";
  my $content = <$fh>;
  close $fh or croak "Error closing $tt_name: $ERRNO";
  return $content;
}

sub test_rendered {
  my ($self, $chunk1, $chunk2) = @_;

  if(!$chunk1) {
    diag q(No chunk1 in test_rendered);
  }

  if(!$chunk2) {
    diag q(No chunk2 in test_rendered);
  }

  if($chunk2 =~ m{^t/}mx) {
    $chunk2 = $self->rendered($chunk2);

    if(!length $chunk2) {
      diag("Zero-sized $chunk2. Expected something like\n$chunk1");
    }
  }

  my $chunk1els = $self->parse_html_to_get_expected($chunk1);
  my $chunk2els = $self->parse_html_to_get_expected($chunk2);
  my $pass      = $self->match_tags($chunk2els, $chunk1els);

  if($pass) {
    return 1;

  } else {
    my ($fn) = $chunk2 =~ m{([^/]+)$}mx;
    $fn     .= q(-);
    open my $fh1, q(>), "/tmp/${fn}chunk1" or croak "Error opening /tmp/${fn}chunk1";
    open my $fh2, q(>), "/tmp/${fn}chunk2" or croak "Error opening /tmp/${fn}chunk2";
    print $fh1 $chunk1;
    print $fh2 $chunk2;
    close $fh1 or croak "Error closing /tmp/${fn}chunk1";
    close $fh2 or croak "Error closing /tmp/${fn}chunk2";
    diag("diff /tmp/${fn}chunk1 /tmp/${fn}chunk2");
  }

  return;
}

sub parse_html_to_get_expected {
  my ($self, $html) = @_;
  my $p;
  my $array = [];

  if ($html =~ m{^t/}xms) {
    $p = HTML::PullParser->new(
			       file  => $html,
			       start => '"S", tagname, @attr',
			       end   => '"E", tagname',
			      );
  } else {
    $p = HTML::PullParser->new(
			       doc   => $html,
			       start => '"S", tagname, @attr',
			       end   => '"E", tagname',
			      );
  }

  my $count = 1;
  while (my $token = $p->get_token()) {
    my $tag = q{};
    for (@{$token}) {
      $_ =~ s/\d{4}-\d{2}-\d{2}/date/xms;
      $_ =~ s/\d{2}:\d{2}:\d{2}/time/xms;
      $tag .= " $_";
    }
    push @{$array}, [$count, $tag];
    $count++;
  }

  return $array;
}

sub match_tags {
  my ($self, $expected, $rendered) = @_;
  my $fail = 0;
  my $a;

  for my $tag (@{$expected}) {
    my @temp = @{$rendered};
    my $match = 0;
    for ($a= 0; $a < @temp;) {
      my $rendered_tag = shift @{$rendered};
      if ($tag->[1] eq $rendered_tag->[1]) {
        $match++;
        $a = scalar @temp;
      } else {
        $a++;
      }
    }

    if (!$match) {
      diag("Failed to match '$tag->[1]'");
      return 0;
    }
  }

  return 1;
}

###########
# for catching emails, so firstly they don't get sent from within a test
# and secondly you could then parse the caught email
#

sub catch_email {
  my ($self, $model) = @_;
  my $sub = sub {
		 my $msg = shift;
        	 push @{$model->{emails}}, $msg->as_string;
		 return;
	        };
  MIME::Lite->send('sub',$sub);
  return;
}

##########
# for parsing emails to get information from them, probably caught emails
#

sub parse_email {
  my ($self, $email) = @_;
  my $parser = MIME::Parser->new();
  $parser->output_to_core(1);
  my $entity = $parser->parse_data($email);
  $self->set_email_annotation($entity->bodyhandle->as_string());
  $self->set_email_subject($entity->head->get('Subject',0));
  $self->set_email_to($entity->head->get('To',0));
  $self->set_email_bcc($entity->head->get('Bcc',0));
  $self->set_email_from($entity->head->get('From',0));
  return;
}

1;
